@@ -1,3 +1,15 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) 2007 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -1440,7 +1452,7 @@ int main(int argc, char *argv[])
 					process_command(&quit);
 			}
 
-			if (pollfd[i].revents & POLLHUP)
+			if (pollfd[i].revents & (POLLHUP | POLLERR | POLLNVAL))
 				client_dead(i);
 		}
 
