@@ -1,6 +1,7 @@
 #!/usr/bin/python
 
-#####
+##
+## Copyright (C) 2008 Red Hat, Inc. All Rights Reserved.
 ##
 ## The Following Agent Has Been Tested On:
 ##
@@ -63,7 +64,7 @@ def set_power_status(conn, options):
 def main():
 	device_opt = [  "help", "version", "agent", "quiet", "verbose", "debug",
 			"action", "ipaddr", "login", "passwd", "passwd_script",
-			"cmd_prompt", "secure", "port", "identity_file" ]
+			"cmd_prompt", "secure", "port" ]
 
 	options = check_input(device_opt, process_input(device_opt))
 
