@@ -1,3 +1,16 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
+**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #ifndef __HEXVIEW_DOT_H__
 #define __HEXVIEW_DOT_H__
 
@@ -8,8 +21,6 @@
 #include <linux/gfs2_ondisk.h>
 #include <string.h>
 
-#include "copyright.cf"
-
 #ifndef TRUE
 #define TRUE 1
 #endif
@@ -63,6 +74,7 @@ EXTERN char estring[1024]; /* edit string */
 EXTERN uint64_t dev_offset INIT(0);
 EXTERN uint64_t max_block INIT(0);
 EXTERN char *buf INIT(NULL);
+EXTERN uint64_t bufsize INIT(4096);
 EXTERN int termlines INIT(30);
 EXTERN int termcols INIT(80);
 EXTERN int insert INIT(0);
@@ -89,7 +101,6 @@ EXTERN WINDOW *wind;
 EXTERN int gfs1 INIT(0);
 EXTERN int editing INIT(0);
 EXTERN uint64_t temp_blk;
-EXTERN uint64_t starting_blk;
 
 struct gfs_jindex {
         uint64_t ji_addr;       /* starting block of the journal */
@@ -269,7 +280,7 @@ EXTERN enum dsp_mode dmode INIT(HEX_MODE);
 #define DIV_RU(x, y) (((x) + (y) - 1) / (y))
 
 #define TITLE1 "gfs2_edit - Global File System Editor (use with extreme caution)"
-#define TITLE2 REDHAT_COPYRIGHT " - Press H for help"
+#define TITLE2 "Copyright (C) 2006 Red Hat, Inc. - Press H for help"
 
 #define COLOR_TITLE     1
 #define COLOR_NORMAL    2
