@@ -1,3 +1,11 @@
+/*
+ * Copyright (C) 2005 Red Hat, Inc.  All rights reserved.
+ *
+ * This copyrighted material is made available to anyone wishing to use,
+ * modify, copy, or redistribute it subject to the terms and conditions
+ * of the GNU General Public License v.2.
+ */
+
 #include "util.h"
 
 extern char *prog_name;
@@ -87,9 +95,6 @@ void add_mtab_entry(struct mount_options *mo)
 
 	read_proc_mounts(mo);
 
-	if (ignore_mtab())
-		return;
-
 	lock_mtab();
 
 	file = fopen("/etc/mtab", "a");
@@ -120,9 +125,6 @@ void del_mtab_entry(struct mount_options *mo)
 	struct stat sbuf;
 	int found = 0;
 
-	if (ignore_mtab())
-		return;
-
 	lock_mtab();
 
 	old_umask = umask(077);
@@ -170,7 +172,10 @@ void del_mtab_entry(struct mount_options *mo)
 		int e = errno;
 		warn("error stating /etc/mtab: %s", strerror(e));
 	} else
-		chown("/etc/mtab.tmp", sbuf.st_uid, sbuf.st_gid);
+		if(chown("/etc/mtab.tmp", sbuf.st_uid, sbuf.st_gid) < 0) {
+			int e = errno;
+			warn("error changing owner of /etc/mtab.tmp: %s", strerror(e));
+		}
 
 	fclose(mtmp);
 	fclose(mtab);
