@@ -13,10 +13,6 @@ static cman_node_t	cman_nodes[MAX_NODES];
 static int		cman_node_count;
 static char		name_buf[CMAN_MAX_NODENAME_LEN+1];
 
-int shutdown_cman(void)
-{
-	cman_leave_cluster(ch_admin, CMAN_LEAVEFLAG_FORCE);
-}
 
 int kill_cman(int nodeid)
 {
@@ -142,13 +138,13 @@ int setup_cman(void)
 
 	ch = cman_init(NULL);
 	if (!ch) {
-		log_print("cman_init error %d %d", (int) ch, errno);
+		log_print("cman_init error %p %d", ch, errno);
 		return -ENOTCONN;
 	}
 
 	ch_admin = cman_admin_init(NULL);
 	if (!ch_admin) {
-		log_print("cman_admin_init error %d %d", (int) ch_admin, errno);
+		log_print("cman_admin_init error %p %d", ch_admin, errno);
 		rv = -ENOTCONN;
 		goto fail1;
 	}
