@@ -1,9 +1,21 @@
+/******************************************************************************
+*******************************************************************************
+**
+**  Copyright (C) 2005 Red Hat, Inc.  All rights reserved.
+**
+**  This copyrighted material is made available to anyone wishing to use,
+**  modify, copy, or redistribute it subject to the terms and conditions
+**  of the GNU General Public License v.2.
+**
+*******************************************************************************
+******************************************************************************/
+
 #include <signal.h>
 #include <time.h>
 
 #include "gd_internal.h"
 
-#define OPTION_STRING			"Dhs:Vv"
+#define OPTION_STRING			"DhVv"
 #define LOCKFILE_NAME			"/var/run/groupd.pid"
 #define LOG_FILE				"/var/log/groupd.log"
 
@@ -94,30 +106,6 @@ static int do_read(int fd, void *buf, size_t count)
    - we won't be in groupd cpg until after we've verified there's no
      local residual gfs/dlm state */
 
-static int ignore_gfs_nolock(char *sysfs_dir, char *table)
-{
-	char path[PATH_MAX];
-	int fd;
-
-	if (!strstr(sysfs_dir, "gfs"))
-		return 0;
-
-	memset(path, 0, PATH_MAX);
-
-	snprintf(path, PATH_MAX, "%s/%s/lock_module/proto_name",
-		 sysfs_dir, table);
-
-	/* lock_nolock doesn't create the "lock_module" dir at all,
-	   so we'll fail to open this */
-
-	fd = open(path, O_RDONLY);
-	if (fd < 0)
-		return 1;
-
-	close(fd);
-	return 0;
-}
-
 static int kernel_instance_count(char *sysfs_dir)
 {
 	char path[PATH_MAX];
@@ -136,9 +124,6 @@ static int kernel_instance_count(char *sysfs_dir)
 		if (de->d_name[0] == '.')
 			continue;
 
-		if (ignore_gfs_nolock(sysfs_dir, de->d_name))
-			continue;
-
 		log_print("found uncontrolled kernel object %s in %s",
 			  de->d_name, sysfs_dir);
 		rv++;
@@ -147,43 +132,14 @@ static int kernel_instance_count(char *sysfs_dir)
 	return rv;
 }
 
-int check_dead_daemons(int ci)
-{
-	group_t *g;
-
-	if (strncmp(client[ci].type, "fence", 5) == 0) {
-		list_for_each_entry(g, &gd_groups, list) {
-			if (client[ci].level == g->level) {
-				return 1;
-			}
-		}
-	}
-
-	if (strncmp(client[ci].type, "dlm", 3) == 0) {
-		list_for_each_entry(g, &gd_groups, list) {
-			if (client[ci].level == g->level) {
-				return 1;
-			}
-		}
-	}
-
-	if (strncmp(client[ci].type, "gfs", 3) == 0) {
-		list_for_each_entry(g, &gd_groups, list) {
-			if (client[ci].level == g->level) {
-				return 1;
-			}
-		}
-	}
-
-	return 0;
-}
-
 int check_uncontrolled_groups(void)
 {
 	pid_t pid;
 	char *argv[4];
 	int status, rv = 0;
 
+	/* FIXME: ignore gfs/gfs2 nolock fs's */
+
 	rv += kernel_instance_count("/sys/kernel/dlm");
 	rv += kernel_instance_count("/sys/fs/gfs");
 	rv += kernel_instance_count("/sys/fs/gfs2");
@@ -452,11 +408,6 @@ static void client_alloc(void)
 
 void client_dead(int ci)
 {
-	if (groupd_shutdown_opt && check_dead_daemons(ci)) {
-		log_print("%s daemon appears to be dead", client[ci].type);
-		shutdown_cman();
-	}
-
 	close(client[ci].fd);
 	client[ci].workfn = NULL;
 	client[ci].fd = -1;
@@ -886,7 +837,10 @@ void daemonize(void)
 	if (pid)
 		exit(EXIT_SUCCESS);
 	setsid();
-	chdir("/");
+	if(chdir("/") < 0) {
+		perror("main: unable to chdir");
+		exit(EXIT_FAILURE);
+	}
 	umask(0);
 	close(0);
 	close(1);
@@ -906,7 +860,6 @@ static void print_usage(void)
 	printf("\n");
 	printf("  -D	       Enable debugging code and don't fork\n");
 	printf("  -h	       Print this help, then exit\n");
-	printf("  -s [0|1]     Enable (or disable) shutdown mode\n");
 	printf("  -V	       Print program version information, then exit\n");
 }
 
@@ -929,10 +882,6 @@ static void decode_arguments(int argc, char **argv)
 			exit(EXIT_SUCCESS);
 			break;
 
-		case 's':
-			groupd_shutdown_opt = atoi(optarg);
-			break;
-
 		case 'v':
 			groupd_debug_verbose++;
 			break;
@@ -1006,8 +955,14 @@ void bail_with_log(int sig)
 		memset(now_ascii, 0, sizeof(now_ascii));
 		time(&now);
 		sprintf(now_ascii, "%ld", now);
-		write(fd, now_ascii, strlen(now_ascii));
-		write(fd, " groupd segfault log follows:\n", 30);
+		if (write(fd, now_ascii, strlen(now_ascii)) < 0) {
+			perror("Unable to write");
+			exit(1);
+		}
+		if (write(fd, " groupd segfault log follows:\n", 30) < 0) {
+			perror("Unable to write");
+			exit(1);
+		}
 		close(fd);
 	} else
 		perror(LOG_FILE);
@@ -1062,7 +1017,6 @@ void groupd_dump_save(void)
 char *prog_name;
 int groupd_debug_opt;
 int groupd_debug_verbose;
-int groupd_shutdown_opt = 1;
 char groupd_debug_buf[256];
 char dump_buf[DUMP_SIZE];
 int dump_point;
