@@ -12,7 +12,8 @@
 
 #include "dlm_daemon.h"
 #include "config.h"
-#include <linux/dlm.h>
+#include <pthread.h>
+#include <linux/dlmconstants.h>
 #include <linux/netlink.h>
 #include <linux/genetlink.h>
 #include <linux/dlm_netlink.h>
@@ -27,6 +28,8 @@ static int client_size = 0;
 static struct client *client = NULL;
 static struct pollfd *pollfd = NULL;
 static int group_mode;
+static pthread_t query_thread;
+static pthread_mutex_t query_mutex;
 
 struct client {
 	int fd;
@@ -73,23 +76,6 @@ int do_write(int fd, void *buf, size_t count)
 	return 0;
 }
 
-static void do_dump(int fd)
-{
-	int len;
-
-	if (dump_wrap) {
-		len = DUMP_SIZE - dump_point;
-		do_write(fd, dump_buf + dump_point, len);
-		len = dump_point;
-	} else
-		len = dump_point;
-
-	/* NUL terminate the debug string */
-	dump_buf[dump_point] = '\0';
-
-	do_write(fd, dump_buf, len);
-}
-
 static void client_alloc(void)
 {
 	int i;
@@ -183,7 +169,7 @@ static struct lockspace *create_ls(char *name)
 	if (!ls)
 		goto out;
 	memset(ls, 0, sizeof(*ls));
-	strncpy(ls->name, name, MAX_LS_NAME);
+	strncpy(ls->name, name, DLM_LOCKSPACE_LEN);
 
 	INIT_LIST_HEAD(&ls->changes);
 	INIT_LIST_HEAD(&ls->node_history);
@@ -380,42 +366,298 @@ static int setup_uevent(void)
 	return s;
 }
 
-/* FIXME: use a library?  Add ability to list lockspaces and their state. */
+static void init_header(struct dlmc_header *h, int cmd, char *name, int result,
+			int extra_len)
+{
+	memset(h, 0, sizeof(struct dlmc_header));
+
+	h->magic = DLMC_MAGIC;
+	h->version = DLMC_VERSION;
+	h->len = sizeof(struct dlmc_header) + extra_len;
+	h->command = cmd;
+	h->data = result;
+
+	if (name)
+		strncpy(h->name, name, DLM_LOCKSPACE_LEN);
+}
+
+static void query_dump_debug(int fd)
+{
+	struct dlmc_header h;
+	int extra_len;
+	int len;
+
+	/* in the case of dump_wrap, extra_len will go in two writes,
+	   first the log tail, then the log head */
+	if (dump_wrap)
+		extra_len = DLMC_DUMP_SIZE;
+	else
+		extra_len = dump_point;
+
+	init_header(&h, DLMC_CMD_DUMP_DEBUG, NULL, 0, extra_len);
+	do_write(fd, &h, sizeof(h));
+
+	if (dump_wrap) {
+		len = DLMC_DUMP_SIZE - dump_point;
+		do_write(fd, dump_buf + dump_point, len);
+		len = dump_point;
+	} else
+		len = dump_point;
+
+	/* NUL terminate the debug string */
+	dump_buf[dump_point] = '\0';
+
+	do_write(fd, dump_buf, len);
+}
+
+static void query_dump_plocks(int fd, char *name)
+{
+	struct lockspace *ls;
+	struct dlmc_header h;
+	int rv;
+
+	ls = find_ls(name);
+	if (!ls) {
+		plock_dump_len = 0;
+		rv = -ENOENT;
+	} else {
+		/* writes to plock_dump_buf and sets plock_dump_len */
+		rv = fill_plock_dump_buf(ls);
+	}
+
+	init_header(&h, DLMC_CMD_DUMP_PLOCKS, name, rv, plock_dump_len);
+
+	do_write(fd, &h, sizeof(h));
+
+	if (plock_dump_len)
+		do_write(fd, plock_dump_buf, plock_dump_len);
+}
+
+/* combines a header and the data and sends it back to the client in
+   a single do_write() call */
+
+static void do_reply(int fd, int cmd, char *name, int result, char *buf,
+		     int buflen)
+{
+	char *reply;
+	int reply_len;
+
+	reply_len = sizeof(struct dlmc_header) + buflen;
+	reply = malloc(reply_len);
+	if (!reply)
+		return;
+	memset(reply, 0, reply_len);
+
+	init_header((struct dlmc_header *)reply, cmd, name, result, buflen);
+
+	if (buf && buflen)
+		memcpy(reply + sizeof(struct dlmc_header), buf, buflen);
+
+	do_write(fd, reply, reply_len);
+
+	free(reply);
+}
+
+static void query_lockspace_info(int fd, char *name)
+{
+	struct lockspace *ls;
+	struct dlmc_lockspace lockspace;
+	int rv;
+
+	ls = find_ls(name);
+	if (!ls) {
+		rv = -ENOENT;
+		goto out;
+	}
+
+	if (group_mode == GROUP_LIBGROUP)
+		rv = set_lockspace_info_group(ls, &lockspace);
+	else
+		rv = set_lockspace_info(ls, &lockspace);
+ out:
+	do_reply(fd, DLMC_CMD_LOCKSPACE_INFO, name, rv,
+		 (char *)&lockspace, sizeof(lockspace));
+}
+
+static void query_node_info(int fd, char *name, int nodeid)
+{
+	struct lockspace *ls;
+	struct dlmc_node node;
+	int rv;
+
+	ls = find_ls(name);
+	if (!ls) {
+		rv = -ENOENT;
+		goto out;
+	}
+
+	if (group_mode == GROUP_LIBGROUP)
+		rv = set_node_info_group(ls, nodeid, &node);
+	else
+		rv = set_node_info(ls, nodeid, &node);
+ out:
+	do_reply(fd, DLMC_CMD_NODE_INFO, name, rv,
+		 (char *)&node, sizeof(node));
+}
+
+static void query_lockspaces(int fd, int max)
+{
+	int ls_count = 0;
+	struct dlmc_lockspace *lss = NULL;
+	int rv, result;
+
+	if (group_mode == GROUP_LIBGROUP)
+		rv = set_lockspaces_group(&ls_count, &lss);
+	else
+		rv = set_lockspaces(&ls_count, &lss);
+
+	if (rv < 0) {
+		result = rv;
+		ls_count = 0;
+		goto out;
+	}
+
+	if (ls_count > max) {
+		result = -E2BIG;
+		ls_count = max;
+	} else {
+		result = ls_count;
+	}
+ out:
+	do_reply(fd, DLMC_CMD_LOCKSPACES, NULL, result,
+		 (char *)lss, ls_count * sizeof(struct dlmc_lockspace));
+
+	if (lss)
+		free(lss);
+}
+
+static void query_lockspace_nodes(int fd, char *name, int option, int max)
+{
+	struct lockspace *ls;
+	int node_count = 0;
+	struct dlmc_node *nodes = NULL;
+	int rv, result;
+
+	ls = find_ls(name);
+	if (!ls) {
+		result = -ENOENT;
+		node_count = 0;
+		goto out;
+	}
+
+	if (group_mode == GROUP_LIBGROUP)
+		rv = set_lockspace_nodes_group(ls, option, &node_count, &nodes);
+	else
+		rv = set_lockspace_nodes(ls, option, &node_count, &nodes);
+
+	if (rv < 0) {
+		result = rv;
+		node_count = 0;
+		goto out;
+	}
+
+	/* node_count is the number of structs copied/returned; the caller's
+	   max may be less than that, in which case we copy as many as they
+	   asked for and return -E2BIG */
+
+	if (node_count > max) {
+		result = -E2BIG;
+		node_count = max;
+	} else {
+		result = node_count;
+	}
+ out:
+	do_reply(fd, DLMC_CMD_LOCKSPACE_NODES, name, result,
+		 (char *)nodes, node_count * sizeof(struct dlmc_node));
+
+	if (nodes)
+		free(nodes);
+}
 
 static void process_connection(int ci)
 {
-	char buf[DLM_CONTROLD_MSGLEN], *argv[MAXARGS];
-	int argc = 0, rv;
+	struct dlmc_header h;
+	char *extra = NULL;
+	int rv, extra_len;
 	struct lockspace *ls;
 
-	memset(buf, 0, sizeof(buf));
-	memset(argv, 0, sizeof(char *) * MAXARGS);
-
-	rv = do_read(client[ci].fd, buf, DLM_CONTROLD_MSGLEN);
+	rv = do_read(client[ci].fd, &h, sizeof(h));
 	if (rv < 0) {
-		log_error("client %d fd %d read error %d %d", ci,
-			  client[ci].fd, rv, errno);
-		client_dead(ci);
-		return;
+		log_debug("connection %d read error %d", ci, rv);
+		goto out;
 	}
 
-	log_debug("ci %d read %s", ci, buf);
+	if (h.magic != DLMC_MAGIC) {
+		log_debug("connection %d magic error %x", ci, h.magic);
+		goto out;
+	}
 
-	get_args(buf, &argc, argv, ' ', 2);
+	if ((h.version & 0xFFFF0000) != (DLMC_VERSION & 0xFFFF0000)) {
+		log_debug("connection %d version error %x", ci, h.version);
+		goto out;
+	}
 
-	if (!strncmp(argv[0], "deadlock_check", 14)) {
-		ls = find_ls(argv[1]);
+	if (h.len > sizeof(h)) {
+		extra_len = h.len - sizeof(h);
+		extra = malloc(extra_len);
+		if (!extra) {
+			log_error("process_connection no mem %d", extra_len);
+			goto out;
+		}
+		memset(extra, 0, extra_len);
+
+		rv = do_read(client[ci].fd, extra, extra_len);
+		if (rv < 0) {
+			log_debug("connection %d extra read error %d", ci, rv);
+			goto out;
+		}
+	}
+
+	switch (h.command) {
+	case DLMC_CMD_FS_REGISTER:
+		ls = find_ls(h.name);
+		if (group_mode == GROUP_LIBGROUP) {
+			rv = -EINVAL;
+		} else if (!ls) {
+			rv = -ENOENT;
+		} else {
+			ls->fs_registered = 1;
+			rv = 0;
+		}
+		do_reply(client[ci].fd, DLMC_CMD_FS_REGISTER, h.name, rv,
+			 NULL, 0);
+		break;
+
+	case DLMC_CMD_FS_UNREGISTER:
+		ls = find_ls(h.name);
+		if (ls)
+			ls->fs_registered = 0;
+		break;
+
+	case DLMC_CMD_FS_NOTIFIED:
+		ls = find_ls(h.name);
+		if (ls)
+			rv = set_fs_notified(ls, h.data);
+		else
+			rv = -ENOENT;
+		do_reply(client[ci].fd, DLMC_CMD_FS_NOTIFIED, h.name, rv,
+			 NULL, 0);
+		break;
+
+	case DLMC_CMD_DEADLOCK_CHECK:
+		ls = find_ls(h.name);
 		if (ls)
 			send_cycle_start(ls);
-		else
-			log_debug("deadlock_check ls name not found");
-	} else if (!strncmp(argv[0], "dump", 4)) {
-		do_dump(client[ci].fd);
-		client_dead(ci);
-	} else if (!strncmp(argv[0], "plocks", 6)) {
-		dump_plocks(argv[1], client[ci].fd);
-		client_dead(ci);
+		break;
+
+	default:
+		log_error("process_connection %d unknown command %d",
+			  ci, h.command);
 	}
+ out:
+	if (extra)
+		free(extra);
+	client_dead(ci);
 }
 
 static void process_listener(int ci)
@@ -433,7 +675,7 @@ static void process_listener(int ci)
 	log_debug("client connection %d fd %d", i, fd);
 }
 
-static int setup_listener(void)
+static int setup_listener(char *sock_path)
 {
 	struct sockaddr_un addr;
 	socklen_t addrlen;
@@ -449,7 +691,7 @@ static int setup_listener(void)
 
 	memset(&addr, 0, sizeof(addr));
 	addr.sun_family = AF_LOCAL;
-	strcpy(&addr.sun_path[1], DLM_CONTROLD_SOCK_PATH);
+	strcpy(&addr.sun_path[1], sock_path);
 	addrlen = sizeof(sa_family_t) + strlen(addr.sun_path+1) + 1;
 
 	rv = bind(s, (struct sockaddr *) &addr, addrlen);
@@ -468,6 +710,94 @@ static int setup_listener(void)
 	return s;
 }
 
+void query_lock(void)
+{
+	pthread_mutex_lock(&query_mutex);
+}
+
+void query_unlock(void)
+{
+	pthread_mutex_unlock(&query_mutex);
+}
+
+/* This is a thread, so we have to be careful, don't call log_ functions.
+   We need a thread to process queries because the main thread may block
+   for long periods when writing to sysfs to stop dlm-kernel (any maybe
+   other places). */
+
+static void *process_queries(void *arg)
+{
+	struct dlmc_header h;
+	int s = *((int *)arg);
+	int f, rv;
+
+	for (;;) {
+		f = accept(s, NULL, NULL);
+
+		rv = do_read(f, &h, sizeof(h));
+		if (rv < 0) {
+			goto out;
+		}
+
+		if (h.magic != DLMC_MAGIC) {
+			goto out;
+		}
+
+		if ((h.version & 0xFFFF0000) != (DLMC_VERSION & 0xFFFF0000)) {
+			goto out;
+		}
+
+		query_lock();
+
+		switch (h.command) {
+		case DLMC_CMD_DUMP_DEBUG:
+			query_dump_debug(f);
+			break;
+		case DLMC_CMD_DUMP_PLOCKS:
+			query_dump_plocks(f, h.name);
+			break;
+		case DLMC_CMD_LOCKSPACE_INFO:
+			query_lockspace_info(f, h.name);
+			break;
+		case DLMC_CMD_NODE_INFO:
+			query_node_info(f, h.name, h.data);
+			break;
+		case DLMC_CMD_LOCKSPACES:
+			query_lockspaces(f, h.data);
+			break;
+		case DLMC_CMD_LOCKSPACE_NODES:
+			query_lockspace_nodes(f, h.name, h.option, h.data);
+			break;
+		default:
+			break;
+		}
+		query_unlock();
+
+ out:
+		close(f);
+	}
+}
+
+static int setup_queries(void)
+{
+	int rv, s;
+
+	rv = setup_listener(DLMC_QUERY_SOCK_PATH);
+	if (rv < 0)
+		return rv;
+	s = rv;
+
+	pthread_mutex_init(&query_mutex, NULL);
+
+	rv = pthread_create(&query_thread, NULL, process_queries, &s);
+	if (rv < 0) {
+		log_error("can't create query thread");
+		close(s);
+		return rv;
+	}
+	return 0;
+}
+
 static void cluster_dead(int ci)
 {
 	log_error("cluster is down, exiting");
@@ -482,7 +812,11 @@ static int loop(void)
 	void (*workfn) (int ci);
 	void (*deadfn) (int ci);
 
-	rv = setup_listener();
+	rv = setup_queries();
+	if (rv < 0)
+		goto out;
+
+	rv = setup_listener(DLMC_SOCK_PATH);
 	if (rv < 0)
 		goto out;
 	client_add(rv, process_listener, NULL);
@@ -570,6 +904,9 @@ static int loop(void)
 			goto out;
 		}
 
+		/* FIXME: lock/unlock around operations that take a while */
+		query_lock();
+
 		for (i = 0; i <= client_maxi; i++) {
 			if (client[i].fd < 0)
 				continue;
@@ -597,6 +934,8 @@ static int loop(void)
 			}
 			poll_timeout = 1000;
 		}
+
+		query_unlock();
 	}
 	rv = 0;
  out:
@@ -646,26 +985,6 @@ static void lockfile(void)
 	}
 }
 
-static void daemonize(void)
-{
-	pid_t pid = fork();
-	if (pid < 0) {
-		perror("main: cannot fork");
-		exit(EXIT_FAILURE);
-	}
-	if (pid)
-		exit(EXIT_SUCCESS);
-	setsid();
-	chdir("/");
-	umask(0);
-	close(0);
-	close(1);
-	close(2);
-	openlog("dlm_controld", LOG_PID, LOG_DAEMON);
-
-	lockfile();
-}
-
 static void print_usage(void)
 {
 	printf("Usage:\n");
@@ -680,26 +999,30 @@ static void print_usage(void)
 	printf("		on: use libgroup, compat with cluster2/stable2/rhel5\n");
 	printf("		off: use libcpg, no backward compatability\n");
 	printf("		Default is %d\n", DEFAULT_GROUPD_COMPAT);
-	printf("  -d <num>	Enable (1) or disable (0) deadlock code\n");
+	printf("  -f <num>	Enable (1) or disable (0) fencing recovery dependency\n");
+	printf("		Default is %d\n", DEFAULT_ENABLE_FENCING);
+	printf("  -q <num>	Enable (1) or disable (0) quorum recovery dependency\n");
+	printf("		Default is %d\n", DEFAULT_ENABLE_QUORUM);
+	printf("  -d <num>	Enable (1) or disable (0) deadlock detection code\n");
 	printf("		Default is %d\n", DEFAULT_ENABLE_DEADLK);
-	printf("  -p <num>	Enable (1) or disable (0) plock code\n");
+	printf("  -p <num>	Enable (1) or disable (0) plock code for cluster fs\n");
 	printf("		Default is %d\n", DEFAULT_ENABLE_PLOCK);
 	printf("  -P		Enable plock debugging\n");
 	printf("  -l <limit>	Limit the rate of plock operations\n");
 	printf("		Default is %d, set to 0 for no limit\n", DEFAULT_PLOCK_RATE_LIMIT);
-	printf("  -o <n>	plock ownership, 1 enable, 0 disable\n");
+	printf("  -o <n>	Enable (1) or disable (0) plock ownership\n");
 	printf("		Default is %d\n", DEFAULT_PLOCK_OWNERSHIP);
-	printf("  -t <ms>	plock drop resources time (milliseconds)\n");
+	printf("  -t <ms>	plock ownership drop resources time (milliseconds)\n");
 	printf("		Default is %u\n", DEFAULT_DROP_RESOURCES_TIME);
-	printf("  -c <num>	plock drop resources count\n");
+	printf("  -c <num>	plock ownership drop resources count\n");
 	printf("		Default is %u\n", DEFAULT_DROP_RESOURCES_COUNT);
-	printf("  -a <ms>	plock drop resources age (milliseconds)\n");
+	printf("  -a <ms>	plock ownership drop resources age (milliseconds)\n");
 	printf("		Default is %u\n", DEFAULT_DROP_RESOURCES_AGE);
 	printf("  -h		Print this help, then exit\n");
 	printf("  -V		Print program version information, then exit\n");
 }
 
-#define OPTION_STRING			"DKg:d:p:Pl:o:t:c:a:hV"
+#define OPTION_STRING			"DKg:f:q:d:p:Pl:o:t:c:a:hV"
 
 static void read_arguments(int argc, char **argv)
 {
@@ -729,6 +1052,16 @@ static void read_arguments(int argc, char **argv)
 			cfgk_debug = 1;
 			break;
 
+		case 'f':
+			optd_enable_fencing = 1;
+			cfgd_enable_fencing = atoi(optarg);
+			break;
+
+		case 'q':
+			optd_enable_quorum = 1;
+			cfgd_enable_quorum = atoi(optarg);
+			break;
+
 		case 'd':
 			optd_enable_deadlk = 1;
 			cfgd_enable_deadlk = atoi(optarg);
@@ -775,7 +1108,8 @@ static void read_arguments(int argc, char **argv)
 			break;
 
 		case 'V':
-			printf("dlm_controld (built %s %s)\n", __DATE__, __TIME__);
+			printf("dlm_controld %s (built %s %s)\n",
+				RELEASE_VERSION, __DATE__, __TIME__);
 			/* printf("%s\n", REDHAT_COPYRIGHT); */
 			exit(EXIT_SUCCESS);
 			break;
@@ -834,8 +1168,15 @@ int main(int argc, char **argv)
 
 	read_arguments(argc, argv);
 
-	if (!daemon_debug_opt)
-		daemonize();
+	lockfile();
+
+	if (!daemon_debug_opt) {
+		if (daemon(0, 0) < 0) {
+			perror("daemon error");
+			exit(EXIT_FAILURE);
+		}
+	}
+	openlog("dlm_controld", LOG_PID, LOG_DAEMON);
 	signal(SIGTERM, sigterm_handler);
 
 	read_ccs();
@@ -867,7 +1208,7 @@ void daemon_dump_save(void)
 	for (i = 0; i < len; i++) {
 		dump_buf[dump_point++] = daemon_debug_buf[i];
 
-		if (dump_point == DUMP_SIZE) {
+		if (dump_point == DLMC_DUMP_SIZE) {
 			dump_point = 0;
 			dump_wrap = 1;
 		}
@@ -886,7 +1227,9 @@ struct list_head lockspaces;
 int cman_quorate;
 int our_nodeid;
 char daemon_debug_buf[256];
-char dump_buf[DUMP_SIZE];
+char dump_buf[DLMC_DUMP_SIZE];
 int dump_point;
 int dump_wrap;
+char plock_dump_buf[DLMC_DUMP_SIZE];
+int plock_dump_len;
 
