@@ -783,15 +783,11 @@ static int add_waiter(struct lockspace *ls, struct resource *r,
 static void write_result(struct lockspace *ls, struct dlm_plock_info *in,
 			 int rv)
 {
-	int err;
-
 	if (need_fsid_translation)
 		in->fsid = ls->associated_mg_id;
 
 	in->rv = rv;
-	err = write(control_fd, in, sizeof(struct dlm_plock_info));
-	if (err != sizeof(struct dlm_plock_info))
-		log_error("plock result write err %d errno %d", err, errno);
+	write(control_fd, in, sizeof(struct dlm_plock_info));
 }
 
 static void do_waiters(struct lockspace *ls, struct resource *r)
@@ -1569,6 +1565,8 @@ void process_plocks(int ci)
 		return;
 	}
 
+	gettimeofday(&now, NULL);
+
 	memset(&info, 0, sizeof(info));
 
 	rv = do_read(control_fd, &info, sizeof(info));
@@ -1645,7 +1643,7 @@ void process_plocks(int ci)
 
  fail:
 	info.rv = rv;
-	write(control_fd, &info, sizeof(info));
+	rv = write(control_fd, &info, sizeof(info));
 }
 
 void process_saved_plocks(struct lockspace *ls)
@@ -2247,25 +2245,20 @@ void purge_plocks(struct lockspace *ls, int nodeid, int unmount)
 	log_group(ls, "purged %d plocks for %d", purged, nodeid);
 }
 
-int dump_plocks(char *name, int fd)
+int fill_plock_dump_buf(struct lockspace *ls)
 {
-	struct lockspace *ls;
 	struct posix_lock *po;
 	struct lock_waiter *w;
 	struct resource *r;
-	char line[MAXLINE];
-	int rv;
+	int rv = 0;
+	int len = DLMC_DUMP_SIZE, pos = 0, ret;
 
-	if (!name)
-		return -1;
-
-	ls = find_ls(name);
-	if (!ls)
-		return -1;
+	memset(plock_dump_buf, 0, sizeof(plock_dump_buf));
+	plock_dump_len = 0;
 
 	list_for_each_entry(r, &ls->plock_resources, list) {
 		list_for_each_entry(po, &r->locks, list) {
-			snprintf(line, MAXLINE,
+			ret = snprintf(plock_dump_buf + pos, len - pos,
 			      "%llu %s %llu-%llu nodeid %d pid %u owner %llx\n",
 			      (unsigned long long)r->number,
 			      po->ex ? "WR" : "RD",
@@ -2274,11 +2267,15 @@ int dump_plocks(char *name, int fd)
 			      po->nodeid, po->pid,
 			      (unsigned long long)po->owner);
 
-			rv = do_write(fd, line, strlen(line));
+			if (ret >= len - pos) {
+				rv = -ENOSPC;
+				goto out;
+			}
+			pos += ret;
 		}
 
 		list_for_each_entry(w, &r->waiters, list) {
-			snprintf(line, MAXLINE,
+			ret = snprintf(plock_dump_buf + pos, len - pos,
 			      "%llu WAITING %s %llu-%llu nodeid %d pid %u owner %llx\n",
 			      (unsigned long long)r->number,
 			      w->info.ex ? "WR" : "RD",
@@ -2287,11 +2284,15 @@ int dump_plocks(char *name, int fd)
 			      w->info.nodeid, w->info.pid,
 			      (unsigned long long)w->info.owner);
 
-			rv = do_write(fd, line, strlen(line));
+			if (ret >= len - pos) {
+				rv = -ENOSPC;
+				goto out;
+			}
+			pos += ret;
 		}
 	}
-
-	return 0;
+ out:
+	return rv;
 }
 
 #if 0
