@@ -617,7 +617,7 @@ int loop(void)
 					process_client(i);
 			}
 
-			if (pollfd[i].revents & POLLHUP) {
+			if (pollfd[i].revents & (POLLHUP | POLLERR | POLLNVAL)) {
 				if (pollfd[i].fd == cman_fd) {
 					log_error("cman connection died");
 					exit_cman();
@@ -781,31 +781,11 @@ static void lockfile(void)
 	}
 }
 
-void daemonize(void)
-{
-	pid_t pid = fork();
-	if (pid < 0) {
-		perror("main: cannot fork");
-		exit(EXIT_FAILURE);
-	}
-	if (pid)
-		exit(EXIT_SUCCESS);
-	setsid();
-	chdir("/");
-	umask(0);
-	close(0);
-	close(1);
-	close(2);
-	openlog("gfs_controld", LOG_PID, LOG_DAEMON);
-
-	lockfile();
-}
-
 static void print_usage(void)
 {
 	printf("Usage:\n");
 	printf("\n");
-	printf("%s [options]\n", prog_name);
+	printf("gfs_controld [options]\n");
 	printf("\n");
 	printf("Options:\n");
 	printf("\n");
@@ -886,7 +866,8 @@ static void decode_arguments(int argc, char **argv)
 			break;
 
 		case 'V':
-			printf("gfs_controld (built %s %s)\n", __DATE__, __TIME__);
+			printf("gfs_controld %s (built %s %s)\n",
+				RELEASE_VERSION, __DATE__, __TIME__);
 			/* printf("%s\n", REDHAT_COPYRIGHT); */
 			exit(EXIT_SUCCESS);
 			break;
@@ -941,8 +922,6 @@ void set_scheduler(void)
 
 int main(int argc, char **argv)
 {
-	prog_name = argv[0];
-
 	INIT_LIST_HEAD(&mounts);
 	INIT_LIST_HEAD(&withdrawn_mounts);
 
@@ -956,8 +935,15 @@ int main(int argc, char **argv)
 
 	decode_arguments(argc, argv);
 
-	if (!daemon_debug_opt)
-		daemonize();
+	lockfile();
+
+	if (!daemon_debug_opt) {
+		if (daemon(0, 0) < 0) {
+			perror("daemon error");
+			exit(EXIT_FAILURE);
+		}
+	}
+	openlog("gfs_controld", LOG_PID, LOG_DAEMON);
 
 	/* ccs settings override the defaults, but not the command line */
 	set_ccs_config();
@@ -999,7 +985,6 @@ void daemon_dump_save(void)
 	}
 }
 
-char *prog_name;
 int plock_debug_opt;
 int daemon_debug_opt;
 char daemon_debug_buf[256];
