@@ -1,3 +1,22 @@
+/*
+  Copyright Red Hat, Inc. 2002
+  Copyright Mission Critical Linux, 2000
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2, or (at your option) any
+  later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; see the file COPYING.  If not, write to the
+  Free Software Foundation, Inc.,  675 Mass Ave, Cambridge, 
+  MA 02139, USA.
+*/
 /** @file
  * Utility for logging arbitrary strings to the cluster log file via syslog.
  *
@@ -64,7 +83,7 @@ main(int argc, char **argv)
     int  opt;
     int  severity = 7,
 	 cmdline_loglevel = 0;/* set if we should not use the config file val*/
-    char *logmsg, *msg_ptr = argv[argc-1];
+    char *logmsg;
     int  pid = 0;
     char *progname = NULL;
     int result;
@@ -73,8 +92,6 @@ main(int argc, char **argv)
     if (argc < 4)
 	usage(argv[0]);
 
-    --argc; /* don't parse last value in getopt */
-
     while ((opt = getopt(argc, argv, "f:l:s:hp:n:")) != -1) {
 	switch (opt) {
 	case 'l':
@@ -101,7 +118,7 @@ main(int argc, char **argv)
 
     /* Add two bytes for linefeed and NULL terminator */
     len = strlen(argv[argc-1]) + 2;
-    logmsg = (char*)malloc(strlen(msg_ptr)+2);
+    logmsg = (char*)malloc(strlen(argv[argc-1])+2);
     if (logmsg == NULL) {
         fprintf(stderr,
             "clulog: malloc fail err=%d\n", errno);
